@@ -36,9 +36,6 @@
 #ifdef USE_MMX
 
 #include <mmintrin.h>
-#ifdef USE_SSE
-#include <xmmintrin.h> /* for _mm_shuffle_pi16 and _MM_SHUFFLE */
-#endif
 
 #include "pixman-mmx.h"
 
@@ -74,15 +71,19 @@
  * possible.
  */
 
-/* --------------- MMX primitivess ------------------------------------ */
+/* --------------- MMX primitives ------------------------------------- */
 
 #ifdef __GNUC__
-typedef unsigned long long ullong;
-typedef ullong mmxdatafield;
-#endif
-#ifdef _MSC_VER
-typedef unsigned __int64 ullong;
+typedef uint64_t mmxdatafield;
+#else
 typedef __m64 mmxdatafield;
+/* If __m64 is defined as a struct or union, define M64_MEMBER to be the
+   name of the member used to access the data */
+# ifdef _MSC_VER
+#  define M64_MEMBER m64_u64
+# elif defined(__SUNPRO_C)
+#  define M64_MEMBER l_
+# endif
 #endif
 
 typedef struct
@@ -104,42 +105,31 @@ typedef struct
     mmxdatafield mmx_000000000000ffff;
 } MMXData;
 
+#if defined(_MSC_VER)
+# define MMXDATA_INIT(field, val) { val##UI64 }
+#elif defined(M64_MEMBER)	/* __m64 is a struct, not an integral type */
+# define MMXDATA_INIT(field, val) field =   { val##ULL }
+#else				/* __m64 is an integral type */
+# define MMXDATA_INIT(field, val) field =   val##ULL
+#endif
+
 static const MMXData c =
 {
-#ifdef __GNUC__
-    .mmx_4x00ff =			0x00ff00ff00ff00ffULL,
-    .mmx_4x0080 =			0x0080008000800080ULL,
-    .mmx_565_rgb =			0x000001f0003f001fULL,
-    .mmx_565_unpack_multiplier =	0x0000008404100840ULL,
-    .mmx_565_r =			0x000000f800000000ULL,
-    .mmx_565_g =			0x0000000000fc0000ULL,
-    .mmx_565_b =			0x00000000000000f8ULL,
-    .mmx_mask_0 =			0xffffffffffff0000ULL,
-    .mmx_mask_1 =			0xffffffff0000ffffULL,
-    .mmx_mask_2 =			0xffff0000ffffffffULL,
-    .mmx_mask_3 =			0x0000ffffffffffffULL,
-    .mmx_full_alpha =			0x00ff000000000000ULL,
-    .mmx_ffff0000ffff0000 =		0xffff0000ffff0000ULL,
-    .mmx_0000ffff00000000 =		0x0000ffff00000000ULL,
-    .mmx_000000000000ffff =		0x000000000000ffffULL,
-#endif
-#ifdef _MSC_VER
-    { 0x00ff00ff00ff00ffUI64 },
-    { 0x0080008000800080UI64 },
-    { 0x000001f0003f001fUI64 },
-    { 0x0000008404100840UI64 },
-    { 0x000000f800000000UI64 },
-    { 0x0000000000fc0000UI64 },
-    { 0x00000000000000f8UI64 },
-    { 0xffffffffffff0000UI64 },
-    { 0xffffffff0000ffffUI64 },
-    { 0xffff0000ffffffffUI64 },
-    { 0x0000ffffffffffffUI64 },
-    { 0x00ff000000000000UI64 },
-    { 0xffff0000ffff0000UI64 },
-    { 0x0000ffff00000000UI64 },
-    { 0x000000000000ffffUI64 },
-#endif
+    MMXDATA_INIT(.mmx_4x00ff,			0x00ff00ff00ff00ff),
+    MMXDATA_INIT(.mmx_4x0080,			0x0080008000800080),
+    MMXDATA_INIT(.mmx_565_rgb,			0x000001f0003f001f),
+    MMXDATA_INIT(.mmx_565_unpack_multiplier,	0x0000008404100840),
+    MMXDATA_INIT(.mmx_565_r,			0x000000f800000000),
+    MMXDATA_INIT(.mmx_565_g,			0x0000000000fc0000),
+    MMXDATA_INIT(.mmx_565_b,			0x00000000000000f8),
+    MMXDATA_INIT(.mmx_mask_0,			0xffffffffffff0000),
+    MMXDATA_INIT(.mmx_mask_1,			0xffffffff0000ffff),
+    MMXDATA_INIT(.mmx_mask_2,			0xffff0000ffffffff),
+    MMXDATA_INIT(.mmx_mask_3,			0x0000ffffffffffff),
+    MMXDATA_INIT(.mmx_full_alpha,		0x00ff000000000000),
+    MMXDATA_INIT(.mmx_ffff0000ffff0000,		0xffff0000ffff0000),
+    MMXDATA_INIT(.mmx_0000ffff00000000,		0x0000ffff00000000),
+    MMXDATA_INIT(.mmx_000000000000ffff,		0x000000000000ffff),
 };
 
 #ifdef __GNUC__
@@ -148,50 +138,39 @@ static const MMXData c =
 #    else
 #        define MC(x) ((__m64)c.mmx_##x)
 #    endif
-#    define inline __inline__ __attribute__ ((__always_inline__))
-#endif
-
-#ifdef _MSC_VER
+#else
 #    define MC(x) c.mmx_##x
-#    undef inline
-#    define inline __forceinline
 #endif
 
-static inline __m64
-M64 (ullong x)
+static force_inline __m64
+M64 (uint64_t x)
 {
 #ifdef __ICC
     return _mm_cvtsi64_m64 (x);
-#elif defined (__GNUC__)
-    return (__m64)x;
-#endif
-
-#ifdef _MSC_VER
+#elif defined M64_MEMBER	/* __m64 is a struct, not an integral type */
     __m64 res;
 
-    res.m64_u64 = x;
+    res.M64_MEMBER = x;
     return res;
+#else				/* __m64 is an integral type */
+    return (__m64)x;
 #endif
 }
 
-static inline ullong
-ULLONG (__m64 x)
+static force_inline uint64_t
+UINT64 (__m64 x)
 {
 #ifdef __ICC
     return _mm_cvtm64_si64 (x);
-#elif defined (__GNUC__)
-    return (ullong)x;
-#endif
-
-#ifdef _MSC_VER
-    ullong res;
-
-    res = x.m64_u64;
+#elif defined M64_MEMBER	/* __m64 is a struct, not an integral type */
+    uint64_t res = x.M64_MEMBER;
     return res;
+#else				/* __m64 is an integral type */
+    return (uint64_t)x;
 #endif
 }
 
-static inline __m64
+static force_inline __m64
 shift (__m64 v, int s)
 {
     if (s > 0)
@@ -202,13 +181,13 @@ shift (__m64 v, int s)
 	return v;
 }
 
-static inline __m64
+static force_inline __m64
 negate (__m64 mask)
 {
     return _mm_xor_si64 (mask, MC(4x00ff));
 }
 
-static inline __m64
+static force_inline __m64
 pix_multiply (__m64 a, __m64 b)
 {
     __m64 res;
@@ -221,35 +200,13 @@ pix_multiply (__m64 a, __m64 b)
     return res;
 }
 
-static inline __m64
+static force_inline __m64
 pix_add (__m64 a, __m64 b)
 {
     return  _mm_adds_pu8 (a, b);
 }
 
-#ifdef USE_SSE
-
-static inline __m64
-expand_alpha (__m64 pixel)
-{
-    return _mm_shuffle_pi16 (pixel, _MM_SHUFFLE(3, 3, 3, 3));
-}
-
-static inline __m64
-expand_alpha_rev (__m64 pixel)
-{
-    return _mm_shuffle_pi16 (pixel, _MM_SHUFFLE(0, 0, 0, 0));
-}
-
-static inline __m64
-invert_colors (__m64 pixel)
-{
-    return _mm_shuffle_pi16 (pixel, _MM_SHUFFLE(3, 0, 1, 2));
-}
-
-#else
-
-static inline __m64
+static force_inline __m64
 expand_alpha (__m64 pixel)
 {
     __m64 t1, t2;
@@ -263,7 +220,7 @@ expand_alpha (__m64 pixel)
     return t1;
 }
 
-static inline __m64
+static force_inline __m64
 expand_alpha_rev (__m64 pixel)
 {
     __m64 t1, t2;
@@ -280,7 +237,7 @@ expand_alpha_rev (__m64 pixel)
     return t1;
 }
 
-static inline __m64
+static force_inline __m64
 invert_colors (__m64 pixel)
 {
     __m64 x, y, z;
@@ -300,15 +257,13 @@ invert_colors (__m64 pixel)
     return x;
 }
 
-#endif
-
-static inline __m64
+static force_inline __m64
 over (__m64 src, __m64 srca, __m64 dest)
 {
     return  _mm_adds_pu8 (src, pix_multiply(dest, negate(srca)));
 }
 
-static inline __m64
+static force_inline __m64
 over_rev_non_pre (__m64 src, __m64 dest)
 {
     __m64 srca = expand_alpha (src);
@@ -317,14 +272,14 @@ over_rev_non_pre (__m64 src, __m64 dest)
     return over(pix_multiply(invert_colors(src), srcfaaa), srca, dest);
 }
 
-static inline __m64
+static force_inline __m64
 in (__m64 src,
     __m64 mask)
 {
     return pix_multiply (src, mask);
 }
 
-static inline __m64
+static force_inline __m64
 in_over_full_src_alpha (__m64 src, __m64 mask, __m64 dest)
 {
     src = _mm_or_si64 (src, MC(full_alpha));
@@ -333,7 +288,7 @@ in_over_full_src_alpha (__m64 src, __m64 mask, __m64 dest)
 }
 
 #ifndef _MSC_VER
-static inline __m64
+static force_inline __m64
 in_over (__m64 src,
 	 __m64 srca,
 	 __m64 mask,
@@ -345,19 +300,19 @@ in_over (__m64 src,
 #define in_over(src, srca, mask, dest) over(in(src, mask), pix_multiply(srca, mask), dest)
 #endif
 
-static inline __m64
+static force_inline __m64
 load8888 (uint32_t v)
 {
     return _mm_unpacklo_pi8 (_mm_cvtsi32_si64 (v), _mm_setzero_si64());
 }
 
-static inline __m64
+static force_inline __m64
 pack8888 (__m64 lo, __m64 hi)
 {
     return _mm_packs_pu16 (lo, hi);
 }
 
-static inline uint32_t
+static force_inline uint32_t
 store8888 (__m64 v)
 {
     return _mm_cvtsi64_si32(pack8888(v, _mm_setzero_si64()));
@@ -377,7 +332,7 @@ store8888 (__m64 v)
  * Note the trick here - the top word is shifted by another nibble to
  * avoid it bumping into the middle word
  */
-static inline __m64
+static force_inline __m64
 expand565 (__m64 pixel, int pos)
 {
     __m64 p = pixel;
@@ -397,7 +352,7 @@ expand565 (__m64 pixel, int pos)
     return _mm_srli_pi16 (pixel, 8);
 }
 
-static inline __m64
+static force_inline __m64
 expand8888 (__m64 in, int pos)
 {
     if (pos == 0)
@@ -406,7 +361,13 @@ expand8888 (__m64 in, int pos)
 	return _mm_unpackhi_pi8 (in, _mm_setzero_si64());
 }
 
-static inline __m64
+static force_inline __m64
+expandx888 (__m64 in, int pos)
+{
+    return _mm_or_si64 (expand8888 (in, pos), MC(full_alpha));
+}
+
+static force_inline __m64
 pack565 (__m64 pixel, __m64 target, int pos)
 {
     __m64 p = pixel;
@@ -437,7 +398,7 @@ pack565 (__m64 pixel, __m64 target, int pos)
 }
 
 #ifndef _MSC_VER
-static inline __m64
+static force_inline __m64
 pix_add_mul (__m64 x, __m64 a, __m64 y, __m64 b)
 {
     x = _mm_mullo_pi16 (x, a);
@@ -461,36 +422,32 @@ pix_add_mul (__m64 x, __m64 a, __m64 y, __m64 b)
 
 /* --------------- MMX code patch for fbcompose.c --------------------- */
 
-static FASTCALL void
-mmxCombineMaskU (uint32_t *src, const uint32_t *mask, int width)
+static force_inline uint32_t
+combine (const uint32_t *src, const uint32_t *mask)
 {
-    const uint32_t *end = mask + width;
-    while (mask < end) {
-        uint32_t mmask = *mask;
-	uint32_t maska = mmask >> 24;
-	if (maska == 0) {
-	    *src = 0;
-	} else if (maska != 0xff) {
-	    __m64 a = load8888(mmask);
-	    __m64 s = load8888(*src);
-	    a = expand_alpha(a);
-	    s = pix_multiply(s, a);
-	    *src = store8888(s);
-	}
-	++src;
-	++mask;
+    uint32_t ssrc = *src;
+
+    if (mask)
+    {
+	__m64 m = load8888 (*mask);
+	__m64 s = load8888 (ssrc);
+
+	m = expand_alpha (m);
+	s = pix_multiply (s, m);
+
+	ssrc = store8888 (s);
     }
-    _mm_empty();
+
+    return ssrc;
 }
 
-
 static FASTCALL void
-mmxCombineOverU (uint32_t *dest, const uint32_t *src, int width)
+mmxCombineOverU (uint32_t *dest, const uint32_t *src, const uint32_t *mask, int width)
 {
     const uint32_t *end = dest + width;
 
     while (dest < end) {
-	uint32_t ssrc = *src;
+	uint32_t ssrc = combine (src, mask);
 	uint32_t a = ssrc >> 24;
 	if (a == 0xff) {
 	    *dest = ssrc;
@@ -502,70 +459,79 @@ mmxCombineOverU (uint32_t *dest, const uint32_t *src, int width)
 	}
 	++dest;
 	++src;
+	if (mask)
+	    ++mask;
     }
     _mm_empty();
 }
 
 static FASTCALL void
-mmxCombineOverReverseU (uint32_t *dest, const uint32_t *src, int width)
+mmxCombineOverReverseU (uint32_t *dest, const uint32_t *src, const uint32_t *mask, int width)
 {
     const uint32_t *end = dest + width;
 
     while (dest < end) {
 	__m64 d, da;
+	uint32_t s = combine (src, mask);
 	d = load8888(*dest);
 	da = expand_alpha(d);
-	*dest = store8888(over (d, da, load8888(*src)));
+	*dest = store8888(over (d, da, load8888(s)));
         ++dest;
         ++src;
+	if (mask)
+	    mask++;
     }
     _mm_empty();
 }
 
 static FASTCALL void
-mmxCombineInU (uint32_t *dest, const uint32_t *src, int width)
+mmxCombineInU (uint32_t *dest, const uint32_t *src, const uint32_t *mask, int width)
 {
     const uint32_t *end = dest + width;
 
     while (dest < end) {
         __m64 x, a;
-        x = load8888(*src);
+        x = load8888 (combine (src, mask));
         a = load8888(*dest);
         a = expand_alpha(a);
         x = pix_multiply(x, a);
         *dest = store8888(x);
         ++dest;
         ++src;
+	if (mask)
+	    mask++;
     }
     _mm_empty();
 }
 
 static FASTCALL void
-mmxCombineInReverseU (uint32_t *dest, const uint32_t *src, int width)
+mmxCombineInReverseU (uint32_t *dest, const uint32_t *src, const uint32_t *mask, int width)
 {
     const uint32_t *end = dest + width;
 
     while (dest < end) {
         __m64 x, a;
         x = load8888(*dest);
-        a = load8888(*src);
+        a = load8888(combine (src, mask));
         a = expand_alpha(a);
         x = pix_multiply(x, a);
         *dest = store8888(x);
         ++dest;
         ++src;
+	if (mask)
+	    mask++;
     }
     _mm_empty();
 }
 
 static FASTCALL void
-mmxCombineOutU (uint32_t *dest, const uint32_t *src, int width)
+mmxCombineOutU (uint32_t *dest, const uint32_t *src, const uint32_t *mask, int width)
 {
     const uint32_t *end = dest + width;
 
     while (dest < end) {
         __m64 x, a;
-        x = load8888(*src);
+        x = load8888(combine (src, mask));
         a = load8888(*dest);
         a = expand_alpha(a);
         a = negate(a);
@@ -573,37 +539,41 @@ mmxCombineOutU (uint32_t *dest, const uint32_t *src, int width)
         *dest = store8888(x);
         ++dest;
         ++src;
+	if (mask)
+	    mask++;
     }
     _mm_empty();
 }
 
 static FASTCALL void
-mmxCombineOutReverseU (uint32_t *dest, const uint32_t *src, int width)
+mmxCombineOutReverseU (uint32_t *dest, const uint32_t *src, const uint32_t *mask, int width)
 {
     const uint32_t *end = dest + width;
 
     while (dest < end) {
         __m64 x, a;
         x = load8888(*dest);
-        a = load8888(*src);
+        a = load8888(combine (src, mask));
         a = expand_alpha(a);
         a = negate(a);
         x = pix_multiply(x, a);
         *dest = store8888(x);
         ++dest;
         ++src;
+	if (mask)
+	    mask++;
     }
     _mm_empty();
 }
 
 static FASTCALL void
-mmxCombineAtopU (uint32_t *dest, const uint32_t *src, int width)
+mmxCombineAtopU (uint32_t *dest, const uint32_t *src, const uint32_t *mask, int width)
 {
     const uint32_t *end = dest + width;
 
     while (dest < end) {
         __m64 s, da, d, sia;
-        s = load8888(*src);
+        s = load8888(combine (src, mask));
         d = load8888(*dest);
         sia = expand_alpha(s);
         sia = negate(sia);
@@ -612,12 +582,14 @@ mmxCombineAtopU (uint32_t *dest, const uint32_t *src, int width)
         *dest = store8888(s);
         ++dest;
         ++src;
+	if (mask)
+	    mask++;
     }
     _mm_empty();
 }
 
 static FASTCALL void
-mmxCombineAtopReverseU (uint32_t *dest, const uint32_t *src, int width)
+mmxCombineAtopReverseU (uint32_t *dest, const uint32_t *src, const uint32_t *mask, int width)
 {
     const uint32_t *end;
 
@@ -625,7 +597,7 @@ mmxCombineAtopReverseU (uint32_t *dest, const uint32_t *src, int width)
 
     while (dest < end) {
         __m64 s, dia, d, sa;
-        s = load8888(*src);
+        s = load8888(combine(src, mask));
         d = load8888(*dest);
         sa = expand_alpha(s);
         dia = expand_alpha(d);
@@ -634,18 +606,20 @@ mmxCombineAtopReverseU (uint32_t *dest, const uint32_t *src, int width)
         *dest = store8888(s);
         ++dest;
         ++src;
+	if (mask)
+	    mask++;
     }
     _mm_empty();
 }
 
 static FASTCALL void
-mmxCombineXorU (uint32_t *dest, const uint32_t *src, int width)
+mmxCombineXorU (uint32_t *dest, const uint32_t *src, const uint32_t *mask, int width)
 {
     const uint32_t *end = dest + width;
 
     while (dest < end) {
         __m64 s, dia, d, sia;
-        s = load8888(*src);
+        s = load8888(combine(src, mask));
         d = load8888(*dest);
         sia = expand_alpha(s);
         dia = expand_alpha(d);
@@ -655,32 +629,36 @@ mmxCombineXorU (uint32_t *dest, const uint32_t *src, int width)
         *dest = store8888(s);
         ++dest;
         ++src;
+	if (mask)
+	    mask++;
     }
     _mm_empty();
 }
 
 static FASTCALL void
-mmxCombineAddU (uint32_t *dest, const uint32_t *src, int width)
+mmxCombineAddU (uint32_t *dest, const uint32_t *src, const uint32_t *mask, int width)
 {
     const uint32_t *end = dest + width;
     while (dest < end) {
         __m64 s, d;
-        s = load8888(*src);
+        s = load8888(combine(src,mask));
         d = load8888(*dest);
         s = pix_add(s, d);
         *dest = store8888(s);
         ++dest;
         ++src;
+	if (mask)
+	    mask++;
     }
     _mm_empty();
 }
 
 static FASTCALL void
-mmxCombineSaturateU (uint32_t *dest, const uint32_t *src, int width)
+mmxCombineSaturateU (uint32_t *dest, const uint32_t *src, const uint32_t *mask, int width)
 {
     const uint32_t *end = dest + width;
     while (dest < end) {
-        uint32_t s = *src;
+        uint32_t s = combine(src,mask);
         uint32_t d = *dest;
         __m64 ms = load8888(s);
         __m64 md = load8888(d);
@@ -696,13 +674,15 @@ mmxCombineSaturateU (uint32_t *dest, const uint32_t *src, int width)
         *dest = store8888(md);
         ++src;
         ++dest;
+	if (mask)
+	    mask++;
     }
     _mm_empty();
 }
 
 
 static FASTCALL void
-mmxCombineSrcC (uint32_t *dest, uint32_t *src, uint32_t *mask, int width)
+mmxCombineSrcC (uint32_t *dest, const uint32_t *src, const uint32_t *mask, int width)
 {
     const uint32_t *end = src + width;
     while (src < end) {
@@ -718,7 +698,7 @@ mmxCombineSrcC (uint32_t *dest, uint32_t *src, uint32_t *mask, int width)
 }
 
 static FASTCALL void
-mmxCombineOverC (uint32_t *dest, uint32_t *src, uint32_t *mask, int width)
+mmxCombineOverC (uint32_t *dest, const uint32_t *src, const uint32_t *mask, int width)
 {
     const uint32_t *end = src + width;
     while (src < end) {
@@ -737,7 +717,7 @@ mmxCombineOverC (uint32_t *dest, uint32_t *src, uint32_t *mask, int width)
 }
 
 static FASTCALL void
-mmxCombineOverReverseC (uint32_t *dest, uint32_t *src, uint32_t *mask, int width)
+mmxCombineOverReverseC (uint32_t *dest, const uint32_t *src, const uint32_t *mask, int width)
 {
     const uint32_t *end = src + width;
     while (src < end) {
@@ -757,7 +737,7 @@ mmxCombineOverReverseC (uint32_t *dest, uint32_t *src, uint32_t *mask, int width
 
 
 static FASTCALL void
-mmxCombineInC (uint32_t *dest, uint32_t *src, uint32_t *mask, int width)
+mmxCombineInC (uint32_t *dest, const uint32_t *src, const uint32_t *mask, int width)
 {
     const uint32_t *end = src + width;
     while (src < end) {
@@ -776,7 +756,7 @@ mmxCombineInC (uint32_t *dest, uint32_t *src, uint32_t *mask, int width)
 }
 
 static FASTCALL void
-mmxCombineInReverseC (uint32_t *dest, uint32_t *src, uint32_t *mask, int width)
+mmxCombineInReverseC (uint32_t *dest, const uint32_t *src, const uint32_t *mask, int width)
 {
     const uint32_t *end = src + width;
     while (src < end) {
@@ -795,7 +775,7 @@ mmxCombineInReverseC (uint32_t *dest, uint32_t *src, uint32_t *mask, int width)
 }
 
 static FASTCALL void
-mmxCombineOutC (uint32_t *dest, uint32_t *src, uint32_t *mask, int width)
+mmxCombineOutC (uint32_t *dest, const uint32_t *src, const uint32_t *mask, int width)
 {
     const uint32_t *end = src + width;
     while (src < end) {
@@ -815,7 +795,7 @@ mmxCombineOutC (uint32_t *dest, uint32_t *src, uint32_t *mask, int width)
 }
 
 static FASTCALL void
-mmxCombineOutReverseC (uint32_t *dest, uint32_t *src, uint32_t *mask, int width)
+mmxCombineOutReverseC (uint32_t *dest, const uint32_t *src, const uint32_t *mask, int width)
 {
     const uint32_t *end = src + width;
     while (src < end) {
@@ -835,7 +815,7 @@ mmxCombineOutReverseC (uint32_t *dest, uint32_t *src, uint32_t *mask, int width)
 }
 
 static FASTCALL void
-mmxCombineAtopC (uint32_t *dest, uint32_t *src, uint32_t *mask, int width)
+mmxCombineAtopC (uint32_t *dest, const uint32_t *src, const uint32_t *mask, int width)
 {
     const uint32_t *end = src + width;
     while (src < end) {
@@ -857,7 +837,7 @@ mmxCombineAtopC (uint32_t *dest, uint32_t *src, uint32_t *mask, int width)
 }
 
 static FASTCALL void
-mmxCombineAtopReverseC (uint32_t *dest, uint32_t *src, uint32_t *mask, int width)
+mmxCombineAtopReverseC (uint32_t *dest, const uint32_t *src, const uint32_t *mask, int width)
 {
     const uint32_t *end = src + width;
     while (src < end) {
@@ -879,7 +859,7 @@ mmxCombineAtopReverseC (uint32_t *dest, uint32_t *src, uint32_t *mask, int width
 }
 
 static FASTCALL void
-mmxCombineXorC (uint32_t *dest, uint32_t *src, uint32_t *mask, int width)
+mmxCombineXorC (uint32_t *dest, const uint32_t *src, const uint32_t *mask, int width)
 {
     const uint32_t *end = src + width;
     while (src < end) {
@@ -902,7 +882,7 @@ mmxCombineXorC (uint32_t *dest, uint32_t *src, uint32_t *mask, int width)
 }
 
 static FASTCALL void
-mmxCombineAddC (uint32_t *dest, uint32_t *src, uint32_t *mask, int width)
+mmxCombineAddC (uint32_t *dest, const uint32_t *src, const uint32_t *mask, int width)
 {
     const uint32_t *end = src + width;
     while (src < end) {
@@ -938,8 +918,8 @@ fbComposeSetupMMX(void)
         pixman_composeFunctions.combineU[PIXMAN_OP_OUT_REVERSE] = mmxCombineOutReverseU;
         pixman_composeFunctions.combineU[PIXMAN_OP_ATOP] = mmxCombineAtopU;
         pixman_composeFunctions.combineU[PIXMAN_OP_ATOP_REVERSE] = mmxCombineAtopReverseU;
-        pixman_composeFunctions.combineU[PIXMAN_OP_XOR] = mmxCombineXorU;
-        pixman_composeFunctions.combineU[PIXMAN_OP_ADD] = mmxCombineAddU;
+        pixman_composeFunctions.combineU[PIXMAN_OP_XOR] = mmxCombineXorU; 
+	pixman_composeFunctions.combineU[PIXMAN_OP_ADD] = mmxCombineAddU;
         pixman_composeFunctions.combineU[PIXMAN_OP_SATURATE] = mmxCombineSaturateU;
 
         pixman_composeFunctions.combineC[PIXMAN_OP_SRC] = mmxCombineSrcC;
@@ -953,8 +933,6 @@ fbComposeSetupMMX(void)
         pixman_composeFunctions.combineC[PIXMAN_OP_ATOP_REVERSE] = mmxCombineAtopReverseC;
         pixman_composeFunctions.combineC[PIXMAN_OP_XOR] = mmxCombineXorC;
         pixman_composeFunctions.combineC[PIXMAN_OP_ADD] = mmxCombineAddC;
-
-        pixman_composeFunctions.combineMaskU = mmxCombineMaskU;
     }
 
     initialized = TRUE;
@@ -1083,10 +1061,10 @@ fbCompositeSolid_nx0565mmx (pixman_op_t op,
 
 	while (w && (unsigned long)dst & 7)
 	{
-	    ullong d = *dst;
+	    uint64_t d = *dst;
 	    __m64 vdest = expand565 (M64(d), 0);
 	    vdest = pack565(over(vsrc, vsrca, vdest), vdest, 0);
-	    *dst = ULLONG(vdest);
+	    *dst = UINT64(vdest);
 
 	    w--;
 	    dst++;
@@ -1113,10 +1091,10 @@ fbCompositeSolid_nx0565mmx (pixman_op_t op,
 
 	while (w)
 	{
-	    ullong d = *dst;
+	    uint64_t d = *dst;
 	    __m64 vdest = expand565 (M64(d), 0);
 	    vdest = pack565(over(vsrc, vsrca, vdest), vdest, 0);
-	    *dst = ULLONG(vdest);
+	    *dst = UINT64(vdest);
 
 	    w--;
 	    dst++;
@@ -1386,36 +1364,36 @@ fbCompositeSrc_x888xnx8888mmx (pixman_op_t op,
 	    __m64 vs7 = *(__m64 *)(src + 14);
 
 	    vd0 = pack8888 (
-		in_over (expand8888 (vs0, 0), srca, vmask, expand8888 (vd0, 0)),
-		in_over (expand8888 (vs0, 1), srca, vmask, expand8888 (vd0, 1)));
+		in_over (expandx888 (vs0, 0), srca, vmask, expand8888 (vd0, 0)),
+		in_over (expandx888 (vs0, 1), srca, vmask, expand8888 (vd0, 1)));
 
 	    vd1 = pack8888 (
-		in_over (expand8888 (vs1, 0), srca, vmask, expand8888 (vd1, 0)),
-		in_over (expand8888 (vs1, 1), srca, vmask, expand8888 (vd1, 1)));
+		in_over (expandx888 (vs1, 0), srca, vmask, expand8888 (vd1, 0)),
+		in_over (expandx888 (vs1, 1), srca, vmask, expand8888 (vd1, 1)));
 
 	    vd2 = pack8888 (
-		in_over (expand8888 (vs2, 0), srca, vmask, expand8888 (vd2, 0)),
-		in_over (expand8888 (vs2, 1), srca, vmask, expand8888 (vd2, 1)));
+		in_over (expandx888 (vs2, 0), srca, vmask, expand8888 (vd2, 0)),
+		in_over (expandx888 (vs2, 1), srca, vmask, expand8888 (vd2, 1)));
 
 	    vd3 = pack8888 (
-		in_over (expand8888 (vs3, 0), srca, vmask, expand8888 (vd3, 0)),
-		in_over (expand8888 (vs3, 1), srca, vmask, expand8888 (vd3, 1)));
+		in_over (expandx888 (vs3, 0), srca, vmask, expand8888 (vd3, 0)),
+		in_over (expandx888 (vs3, 1), srca, vmask, expand8888 (vd3, 1)));
 
 	    vd4 = pack8888 (
-		in_over (expand8888 (vs4, 0), srca, vmask, expand8888 (vd4, 0)),
-		in_over (expand8888 (vs4, 1), srca, vmask, expand8888 (vd4, 1)));
+		in_over (expandx888 (vs4, 0), srca, vmask, expand8888 (vd4, 0)),
+		in_over (expandx888 (vs4, 1), srca, vmask, expand8888 (vd4, 1)));
 
 	    vd5 = pack8888 (
-		in_over (expand8888 (vs5, 0), srca, vmask, expand8888 (vd5, 0)),
-		in_over (expand8888 (vs5, 1), srca, vmask, expand8888 (vd5, 1)));
+		in_over (expandx888 (vs5, 0), srca, vmask, expand8888 (vd5, 0)),
+		in_over (expandx888 (vs5, 1), srca, vmask, expand8888 (vd5, 1)));
 
             vd6 = pack8888 (
-		in_over (expand8888 (vs6, 0), srca, vmask, expand8888 (vd6, 0)),
-		in_over (expand8888 (vs6, 1), srca, vmask, expand8888 (vd6, 1)));
+		in_over (expandx888 (vs6, 0), srca, vmask, expand8888 (vd6, 0)),
+		in_over (expandx888 (vs6, 1), srca, vmask, expand8888 (vd6, 1)));
 
 	    vd7 = pack8888 (
-		in_over (expand8888 (vs7, 0), srca, vmask, expand8888 (vd7, 0)),
-		in_over (expand8888 (vs7, 1), srca, vmask, expand8888 (vd7, 1)));
+		in_over (expandx888 (vs7, 0), srca, vmask, expand8888 (vd7, 0)),
+		in_over (expandx888 (vs7, 1), srca, vmask, expand8888 (vd7, 1)));
 
 	    *(__m64 *)(dst + 0) = vd0;
 	    *(__m64 *)(dst + 2) = vd1;
@@ -1541,12 +1519,12 @@ fbCompositeSrc_8888x0565mmx (pixman_op_t op,
 	while (w && (unsigned long)dst & 7)
 	{
 	    __m64 vsrc = load8888 (*src);
-	    ullong d = *dst;
+	    uint64_t d = *dst;
 	    __m64 vdest = expand565 (M64(d), 0);
 
 	    vdest = pack565(over(vsrc, expand_alpha(vsrc), vdest), vdest, 0);
 
-	    *dst = ULLONG(vdest);
+	    *dst = UINT64(vdest);
 
 	    w--;
 	    dst++;
@@ -1584,12 +1562,12 @@ fbCompositeSrc_8888x0565mmx (pixman_op_t op,
 	while (w)
 	{
 	    __m64 vsrc = load8888 (*src);
-	    ullong d = *dst;
+	    uint64_t d = *dst;
 	    __m64 vdest = expand565 (M64(d), 0);
 
 	    vdest = pack565(over(vsrc, expand_alpha(vsrc), vdest), vdest, 0);
 
-	    *dst = ULLONG(vdest);
+	    *dst = UINT64(vdest);
 
 	    w--;
 	    dst++;
@@ -1620,7 +1598,7 @@ fbCompositeSolidMask_nx8x8888mmx (pixman_op_t op,
     int	dstStride, maskStride;
     uint16_t	w;
     __m64	vsrc, vsrca;
-    ullong	srcsrc;
+    uint64_t	srcsrc;
 
     CHECKPOINT();
 
@@ -1630,7 +1608,7 @@ fbCompositeSolidMask_nx8x8888mmx (pixman_op_t op,
     if (srca == 0)
 	return;
 
-    srcsrc = (ullong)src << 32 | src;
+    srcsrc = (uint64_t)src << 32 | src;
 
     fbComposeGetStart (pDst, xDst, yDst, uint32_t, dstStride, dstLine, 1);
     fbComposeGetStart (pMask, xMask, yMask, uint8_t, maskStride, maskLine, 1);
@@ -1650,7 +1628,7 @@ fbCompositeSolidMask_nx8x8888mmx (pixman_op_t op,
 
 	while (w && (unsigned long)dst & 7)
 	{
-	    ullong m = *mask;
+	    uint64_t m = *mask;
 
 	    if (m)
 	    {
@@ -1667,13 +1645,13 @@ fbCompositeSolidMask_nx8x8888mmx (pixman_op_t op,
 
 	while (w >= 2)
 	{
-	    ullong m0, m1;
+	    uint64_t m0, m1;
 	    m0 = *mask;
 	    m1 = *(mask + 1);
 
 	    if (srca == 0xff && (m0 & m1) == 0xff)
 	    {
-		*(ullong *)dst = srcsrc;
+		*(uint64_t *)dst = srcsrc;
 	    }
 	    else if (m0 | m1)
 	    {
@@ -1697,7 +1675,7 @@ fbCompositeSolidMask_nx8x8888mmx (pixman_op_t op,
 
 	while (w)
 	{
-	    ullong m = *mask;
+	    uint64_t m = *mask;
 
 	    if (m)
 	    {
@@ -1725,7 +1703,7 @@ pixman_fill_mmx (uint32_t *bits,
 		 int height,
 		 uint32_t xor)
 {
-    ullong	fill;
+    uint64_t	fill;
     __m64	vfill;
     uint32_t	byte_width;
     uint8_t	*byte_line;
@@ -1768,7 +1746,7 @@ pixman_fill_mmx (uint32_t *bits,
 	stride *= 4;
     }
 
-    fill = ((ullong)xor << 32) | xor;
+    fill = ((uint64_t)xor << 32) | xor;
     vfill = M64(fill);
 
 #ifdef __GNUC__
@@ -1891,7 +1869,7 @@ fbCompositeSolidMaskSrc_nx8x8888mmx (pixman_op_t op,
     int	dstStride, maskStride;
     uint16_t	w;
     __m64	vsrc, vsrca;
-    ullong	srcsrc;
+    uint64_t	srcsrc;
 
     CHECKPOINT();
 
@@ -1905,7 +1883,7 @@ fbCompositeSolidMaskSrc_nx8x8888mmx (pixman_op_t op,
 	return;
     }
 
-    srcsrc = (ullong)src << 32 | src;
+    srcsrc = (uint64_t)src << 32 | src;
 
     fbComposeGetStart (pDst, xDst, yDst, uint32_t, dstStride, dstLine, 1);
     fbComposeGetStart (pMask, xMask, yMask, uint8_t, maskStride, maskLine, 1);
@@ -1925,7 +1903,7 @@ fbCompositeSolidMaskSrc_nx8x8888mmx (pixman_op_t op,
 
 	while (w && (unsigned long)dst & 7)
 	{
-	    ullong m = *mask;
+	    uint64_t m = *mask;
 
 	    if (m)
 	    {
@@ -1946,13 +1924,13 @@ fbCompositeSolidMaskSrc_nx8x8888mmx (pixman_op_t op,
 
 	while (w >= 2)
 	{
-	    ullong m0, m1;
+	    uint64_t m0, m1;
 	    m0 = *mask;
 	    m1 = *(mask + 1);
 
 	    if (srca == 0xff && (m0 & m1) == 0xff)
 	    {
-		*(ullong *)dst = srcsrc;
+		*(uint64_t *)dst = srcsrc;
 	    }
 	    else if (m0 | m1)
 	    {
@@ -1968,7 +1946,7 @@ fbCompositeSolidMaskSrc_nx8x8888mmx (pixman_op_t op,
 	    }
 	    else
 	    {
-		*(ullong *)dst = 0;
+		*(uint64_t *)dst = 0;
 	    }
 
 	    mask += 2;
@@ -1980,7 +1958,7 @@ fbCompositeSolidMaskSrc_nx8x8888mmx (pixman_op_t op,
 
 	while (w)
 	{
-	    ullong m = *mask;
+	    uint64_t m = *mask;
 
 	    if (m)
 	    {
@@ -2022,7 +2000,7 @@ fbCompositeSolidMask_nx8x0565mmx (pixman_op_t op,
     int	dstStride, maskStride;
     uint16_t	w;
     __m64	vsrc, vsrca, tmp;
-    ullong srcsrcsrcsrc, src16;
+    uint64_t srcsrcsrcsrc, src16;
 
     CHECKPOINT();
 
@@ -2039,10 +2017,10 @@ fbCompositeSolidMask_nx8x0565mmx (pixman_op_t op,
     vsrca = expand_alpha (vsrc);
 
     tmp = pack565(vsrc, _mm_setzero_si64(), 0);
-    src16 = ULLONG(tmp);
+    src16 = UINT64(tmp);
 
-    srcsrcsrcsrc = (ullong)src16 << 48 | (ullong)src16 << 32 |
-	(ullong)src16 << 16 | (ullong)src16;
+    srcsrcsrcsrc = (uint64_t)src16 << 48 | (uint64_t)src16 << 32 |
+	(uint64_t)src16 << 16 | (uint64_t)src16;
 
     while (height--)
     {
@@ -2056,15 +2034,15 @@ fbCompositeSolidMask_nx8x0565mmx (pixman_op_t op,
 
 	while (w && (unsigned long)dst & 7)
 	{
-	    ullong m = *mask;
+	    uint64_t m = *mask;
 
 	    if (m)
 	    {
-		ullong d = *dst;
+		uint64_t d = *dst;
 		__m64 vd = M64(d);
 		__m64 vdest = in_over(vsrc, vsrca, expand_alpha_rev (M64 (m)), expand565(vd, 0));
 		vd = pack565(vdest, _mm_setzero_si64(), 0);
-		*dst = ULLONG(vd);
+		*dst = UINT64(vd);
 	    }
 
 	    w--;
@@ -2076,7 +2054,7 @@ fbCompositeSolidMask_nx8x0565mmx (pixman_op_t op,
 
 	while (w >= 4)
 	{
-	    ullong m0, m1, m2, m3;
+	    uint64_t m0, m1, m2, m3;
 	    m0 = *mask;
 	    m1 = *(mask + 1);
 	    m2 = *(mask + 2);
@@ -2084,7 +2062,7 @@ fbCompositeSolidMask_nx8x0565mmx (pixman_op_t op,
 
 	    if (srca == 0xff && (m0 & m1 & m2 & m3) == 0xff)
 	    {
-		*(ullong *)dst = srcsrcsrcsrc;
+		*(uint64_t *)dst = srcsrcsrcsrc;
 	    }
 	    else if (m0 | m1 | m2 | m3)
 	    {
@@ -2114,15 +2092,15 @@ fbCompositeSolidMask_nx8x0565mmx (pixman_op_t op,
 
 	while (w)
 	{
-	    ullong m = *mask;
+	    uint64_t m = *mask;
 
 	    if (m)
 	    {
-		ullong d = *dst;
+		uint64_t d = *dst;
 		__m64 vd = M64(d);
 		__m64 vdest = in_over(vsrc, vsrca, expand_alpha_rev (M64(m)), expand565(vd, 0));
 		vd = pack565(vdest, _mm_setzero_si64(), 0);
-		*dst = ULLONG(vd);
+		*dst = UINT64(vd);
 	    }
 
 	    w--;
@@ -2176,12 +2154,12 @@ fbCompositeSrc_8888RevNPx0565mmx (pixman_op_t op,
 	while (w && (unsigned long)dst & 7)
 	{
 	    __m64 vsrc = load8888 (*src);
-	    ullong d = *dst;
+	    uint64_t d = *dst;
 	    __m64 vdest = expand565 (M64(d), 0);
 
 	    vdest = pack565(over_rev_non_pre(vsrc, vdest), vdest, 0);
 
-	    *dst = ULLONG(vdest);
+	    *dst = UINT64(vdest);
 
 	    w--;
 	    dst++;
@@ -2237,12 +2215,12 @@ fbCompositeSrc_8888RevNPx0565mmx (pixman_op_t op,
 	while (w)
 	{
 	    __m64 vsrc = load8888 (*src);
-	    ullong d = *dst;
+	    uint64_t d = *dst;
 	    __m64 vdest = expand565 (M64(d), 0);
 
 	    vdest = pack565(over_rev_non_pre(vsrc, vdest), vdest, 0);
 
-	    *dst = ULLONG(vdest);
+	    *dst = UINT64(vdest);
 
 	    w--;
 	    dst++;
@@ -2306,7 +2284,7 @@ fbCompositeSrc_8888RevNPx8888mmx (pixman_op_t op,
 
 	while (w >= 2)
 	{
-	    ullong s0, s1;
+	    uint64_t s0, s1;
 	    unsigned char a0, a1;
 	    __m64 d0, d1;
 
@@ -2400,10 +2378,10 @@ fbCompositeSolidMask_nx8888x0565Cmmx (pixman_op_t op,
 
 	    if (m)
 	    {
-		ullong d = *q;
+		uint64_t d = *q;
 		__m64 vdest = expand565 (M64(d), 0);
 		vdest = pack565 (in_over (vsrc, vsrca, load8888 (m), vdest), vdest, 0);
-		*q = ULLONG(vdest);
+		*q = UINT64(vdest);
 	    }
 
 	    twidth--;
@@ -2443,10 +2421,10 @@ fbCompositeSolidMask_nx8888x0565Cmmx (pixman_op_t op,
 	    m = *(uint32_t *)p;
 	    if (m)
 	    {
-		ullong d = *q;
+		uint64_t d = *q;
 		__m64 vdest = expand565(M64(d), 0);
 		vdest = pack565 (in_over(vsrc, vsrca, load8888(m), vdest), vdest, 0);
-		*q = ULLONG(vdest);
+		*q = UINT64(vdest);
 	    }
 
 	    twidth--;
@@ -2804,7 +2782,7 @@ fbCompositeSrcAdd_8888x8888mmx (pixman_op_t 	op,
 	while (w >= 2)
 	{
 	    dst64 = _mm_adds_pu8(*(__m64*)src, *(__m64*)dst);
-	    *(ullong*)dst = ULLONG(dst64);
+	    *(uint64_t*)dst = UINT64(dst64);
 	    dst += 2;
 	    src += 2;
 	    w -= 2;
@@ -2888,7 +2866,7 @@ pixman_blt_mmx (uint32_t *src_bits,
 
 	while (w >= 64)
 	{
-#ifdef __GNUC__
+#if defined (__GNUC__) || (defined(__SUNPRO_C) && (__SUNPRO_C >= 0x590))
 	    __asm__ (
 		"movq	  (%1),	  %%mm0\n"
 		"movq	 8(%1),	  %%mm1\n"
@@ -3017,7 +2995,7 @@ fbCompositeOver_x888x8x8888mmx (pixman_op_t      op,
 
 	while (w--)
 	{
-	    ullong m = *mask;
+	    uint64_t m = *mask;
 
 	    if (m)
 	    {
